﻿using System.IO;
using System.IO.Compression;
using System.Reflection;

// ReSharper disable once CheckNamespace
namespace Loader
{
    // This code is public domain.
    //
    // The MurmurHash3 algorithm was created by Austin Appleby and put into the public domain.  See http://code.google.com/p/smhasher/
    //
    // This C# variant was authored by
    // Elliott B. Edwards and was placed into the public domain as a gist
    // Status...Working on verification (Test Suite)

    public static class MurMurHash3
    {
        //Change to suit your needs
        private const uint Seed = 233;

        public static int Hash(Stream stream)
        {
            const uint c1 = 0xcc9e2d51;
            const uint c2 = 0x1b873593;

            var h1 = Seed;
            uint k1;
            uint streamLength = 0;

            using (var reader = new BinaryReader(stream))
            {
                var chunk = reader.ReadBytes(4);
                while (chunk.Length > 0)
                {
                    streamLength += (uint)chunk.Length;
                    switch (chunk.Length)
                    {
                        case 4:
                            /* Get four bytes from the input into an uint */
                            k1 = (uint)
                                (chunk[0]
                                 | chunk[1] << 8
                                 | chunk[2] << 16
                                 | chunk[3] << 24);

                            /* bitmagic hash */
                            k1 *= c1;
                            k1 = Rotl32(k1, 15);
                            k1 *= c2;

                            h1 ^= k1;
                            h1 = Rotl32(h1, 13);
                            h1 = h1 * 5 + 0xe6546b64;
                            break;

                        case 3:
                            k1 = (uint)
                                (chunk[0]
                                 | chunk[1] << 8
                                 | chunk[2] << 16);
                            k1 *= c1;
                            k1 = Rotl32(k1, 15);
                            k1 *= c2;
                            h1 ^= k1;
                            break;

                        case 2:
                            k1 = (uint)
                                (chunk[0]
                                 | chunk[1] << 8);
                            k1 *= c1;
                            k1 = Rotl32(k1, 15);
                            k1 *= c2;
                            h1 ^= k1;
                            break;

                        case 1:
                            k1 = chunk[0];
                            k1 *= c1;
                            k1 = Rotl32(k1, 15);
                            k1 *= c2;
                            h1 ^= k1;
                            break;
                    }
                    chunk = reader.ReadBytes(4);
                }
            }

            // finalization, magic chants to wrap it all up
            h1 ^= streamLength;
            h1 = Fmix(h1);

            unchecked //ignore overflow
            {
                return (int)h1;
            }
        }

        private static uint Rotl32(uint x, byte r)
        {
            return (x << r) | (x >> (32 - r));
        }

        private static uint Fmix(uint h)
        {
            h ^= h >> 16;
            h *= 0x85ebca6b;
            h ^= h >> 13;
            h *= 0xc2b2ae35;
            h ^= h >> 16;
            return h;
        }
    }

    static partial class Program
    {
        public static byte[] GzipCompress(this byte[] data)
        {
            using (MemoryStream msS = new MemoryStream(data), msD = new MemoryStream())
            {
                using (var gz = new GZipStream(msD, CompressionLevel.Optimal))
                {
                    msS.CopyTo(gz);
                }
                return msD.ToArray();
            }
        }

        public static byte[] GzipDecompress(this byte[] data)
        {
            using (MemoryStream msS = new MemoryStream(data), msD = new MemoryStream())
            {
                var gz = new GZipStream(msS, CompressionMode.Decompress);
                gz.CopyTo(msD);
                return msD.ToArray();
            }
        }

        public static string HashData(byte[] data)
        {
            using (var ms = new MemoryStream(data))
            {
                return MurMurHash3.Hash(ms).ToString("X8");
            }
        }

        private static byte[] ToByteArray(this Stream stream)
        {
            var buf = new byte[stream.Length];
            stream.Read(buf, 0, buf.Length);
            return buf;
        }

        // ReSharper disable UnusedMember.Local

        private static byte[] GetResource(string name)
        {
            return Assembly.GetExecutingAssembly().GetManifestResourceStream(name).ToByteArray();
        }

        // ReSharper restore UnusedMember.Local
    }
}